#include <stdio.h>
#include "db_system.h"

/* InitFreeSpace - initialize free space allocator */
void InitFreeSpace(System *sys, uint8_t *space, size_t size)
{
    sys->freeSpace = sys->freeMark = sys->freeNext = space;
    sys->freeTop = space + size;
}

/* MarkFreeSpace - initialize free space allocator */
void MarkFreeSpace(System *sys)
{
    sys->freeMark = sys->freeNext;
}

/* ResetFreeSpace - initialize free space allocator */
void ResetFreeSpace(System *sys)
{
    sys->freeNext = sys->freeMark;
}

/* AllocateFreeSpace - allocate free space */
uint8_t *AllocateFreeSpace(System *sys, size_t size)
{
    uint8_t *p = sys->freeNext;
    size = (size + ALIGN_MASK) & ~ALIGN_MASK;
	if (p + size > sys->freeTop)
        return NULL;
    sys->freeNext += size;
    return p;
}

/* AllocateAllFreeSpace - allocate all of the remaining free space */
uint8_t *AllocateAllFreeSpace(System *sys, size_t *pSize)
{
    uint8_t *p = sys->freeNext;
    *pSize = sys->freeTop - p;
    sys->freeNext = sys->freeTop;
    return p;
}
