/* db_types.c - type functions
 *
 * Copyright (c) 2011 by David Michael Betz.  All rights reserved.
 *
 */

#include <stdlib.h>
#include <string.h>
#include "db_compiler.h"

/* local variables */
Type integerType = { TYPE_INTEGER };
Type stringType = { TYPE_STRING };

/* NewGlobalType - allocate a new global type */
Type *NewGlobalType(ParseContext *c, TypeID id)
{
    Type *type = (Type *)GlobalAlloc(c, sizeof(Type));
    type->id = id;
    return type;
}

/* NewLocalType - allocate a new type */
Type *NewLocalType(ParseContext *c, TypeID id)
{
    Type *type = (Type *)LocalAlloc(c, sizeof(Type));
    type->id = id;
    return type;
}

/* DefaultType - get the default type based on the variable name */
Type *DefaultType(char *name)
{
    Type *type;
    if (name[strlen(name) - 1] == '$')
        type = &stringType;
    else
        type = &integerType;
    return type;
}

/* CompareTypes - compare two types to see if they are equivilent */
int CompareTypes(Type *type1, Type *type2)
{
    int match;

    /* make sure the type ids match */
    if (type1->id == type2->id) {
        switch (type1->id) {
        case TYPE_ARRAY:
            match = CompareTypes(type1->u.arrayInfo.elementType, type2->u.arrayInfo.elementType);
            break;
        default:
            match = VMTRUE;
            break;
        }
    }

    /* no type id match */
    else
        match = VMFALSE;

    return match;
}
