/* db_types.h - type definitions for a simple virtual machine
 *
 * Copyright (c) 2011 by David Michael Betz.  All rights reserved.
 *
 */

#ifndef __DB_TYPES_H__
#define __DB_TYPES_H__

#ifdef PIC24H
#define WORD_SIZE_16
#define NO_STDINT
#define FAR_DATA		__attribute__((far))
#define FLASH_SPACE		const
#define BUFSIZE         1500
#define DATSIZE         1500
#define RAMSIZE         4500
#define MAX_OBJECTS     32
#else
#define WORD_SIZE_32
#define FAR_DATA
#define FLASH_SPACE		const
#define BUFSIZE         2048
#define DATSIZE         8192
#define RAMSIZE         (18 * 1024)
#define MAX_OBJECTS     32
#endif

#ifdef NO_STDINT
typedef long int32_t;
typedef unsigned long uint32_t;
typedef short int16_t;
typedef unsigned short uint16_t;
typedef char int8_t;
typedef unsigned char uint8_t;
#else
#include <stdint.h>
#endif

#ifdef WORD_SIZE_16
typedef int16_t VMVALUE;
typedef uint16_t VMUVALUE;
#endif

#ifdef WORD_SIZE_32
typedef int32_t VMVALUE;
typedef uint32_t VMUVALUE;
#endif

#define VMTRUE		1
#define VMFALSE   	0

int strcasecmp(const char *s1, const char *s2);

#define ALIGN_MASK				(sizeof(VMVALUE) - 1)

#define VMCODEBYTE(p)           *(p)
#define VMINTRINSIC(i)          Intrinsics[i]

#endif
